/*
 * This program is copyright (c) 2007 Hortis-GRC SA.
 *
 * This file is part of Sonar.
 * Sonar is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
package ch.hortis.sonar.model;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;

@Entity
@Table(name = "files")
public class File implements Collectable {

  @Id
  @Column(name = "id")
  @SequenceGenerator(name = "FILES_SEQ", sequenceName = "FILES_SEQ")
  @GeneratedValue(strategy = GenerationType.AUTO, generator = "FILES_SEQ")
  private Integer id;

  @Column(name = "filename", updatable = false, nullable = false, length = 255)
  private String filename;

  @Column(name = "namespace", updatable = false, nullable = true, length = 500)
  private String namespace;

  @Column(name = "path", updatable = false, nullable = true, length = 500)
  private String path;

  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "snapshot_id")
  private Snapshot snapshot;

  @OneToOne(mappedBy = "file", fetch = FetchType.LAZY, cascade = {CascadeType.ALL})
  public FileSource fileSource;

  public File(String filename, String namespace, String path) {
    setFilename(filename);
    this.namespace = namespace;
    this.path = path;
  }

  public File(int id, String filename, String namespace, String path) {
    this(filename, namespace, path);
    setId(id);
  }

  public File() {
  }

  public Snapshot getSnapshot() {
    return snapshot;
  }

  public void setSnapshot(Snapshot snapshot) {
    this.snapshot = snapshot;
  }

  public Integer getId() {
    return id;
  }

  public void setId(Integer id) {
    this.id = id;
  }

  public String getFilename() {
    return filename;
  }

  public void setFilename(String filename) {
    String prefix = StringUtils.substringBeforeLast(filename, ".");
    String suffix = StringUtils.substringAfterLast(filename, ".");
    this.filename = prefix + "." + suffix.toLowerCase();
  }

  public String getPath() {
    return path;
  }

  public void setPath(String path) {
    this.path = path;
  }

  public String getNamespace() {
    return namespace;
  }

  public void setNamespace(String namespace) {
    this.namespace = namespace;
  }

  public FileSource getFileSource() {
    return fileSource;
  }

  public void setFileSource(FileSource fileSource) {
    this.fileSource = fileSource;
  }

  public boolean equals(Object obj) {
    if (!(obj instanceof File)) {
      return false;
    }
    if (this == obj) {
      return true;
    }
    File other = (File) obj;
    return new EqualsBuilder()
        .append(id, other.getId()).isEquals();
  }

  public int hashCode() {
    return new HashCodeBuilder(17, 37).
        append(id).toHashCode();
  }

  public String toString() {
    return id + ":".concat(namespace).concat(":").concat(filename);
  }

}
