/*
 * This program is copyright (c) 2007 Hortis-GRC SA.
 * 
 * This file is part of Sonar.
 * Sonar is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * Sonar is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with Sonar; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
package ch.hortis.sonar.service;

import ch.hortis.sonar.model.File;
import ch.hortis.sonar.model.FileMeasure;
import ch.hortis.sonar.model.Metric;
import ch.hortis.sonar.model.ProjectMeasure;
import ch.hortis.sonar.model.Rule;
import ch.hortis.sonar.model.RulesCategory;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;

public class MeasureKey implements Cloneable {
  protected File file;
  protected Metric metric;
  protected RulesCategory rulesCategory;
  protected Rule rule;
  protected String subkey;

  public MeasureKey(Metric metric, RulesCategory rulesCategory, Rule rule, File file, String subkey) {
    this.file = file;
    this.metric = metric;
    this.rulesCategory = rulesCategory;
    this.rule = rule;
    this.subkey = subkey;
  }

  public MeasureKey(Metric metric) {
    this(metric, null, null, null, null);
  }

  public MeasureKey(Metric metric, RulesCategory rulesCategory, Rule rule) {
    this(metric, rulesCategory, rule, null, null);
  }

  public MeasureKey(Metric metric, RulesCategory rulesCategory, Rule rule, File file) {
    this(metric, rulesCategory, rule, file, null);
  }

  public MeasureKey(FileMeasure measure) {
    this(measure.getMetric(), measure.getRulesCategory(), measure.getRule(), measure.getFile(), null);
  }

  public MeasureKey(ProjectMeasure measure) {
    this(measure.getMetric(), measure.getRulesCategory(), measure.getRule(), null, measure.getSubkey());
  }

  public boolean equals(Object obj) {
    if (!(obj instanceof MeasureKey)) {
      return false;
    }
    if (this == obj) {
      return true;
    }
    MeasureKey other = (MeasureKey) obj;
    return new EqualsBuilder()
        .append(file, other.file)
        .append(metric, other.metric)
        .append(rulesCategory, other.rulesCategory)
        .append(rule, other.rule)
        .append(subkey, other.subkey)
        .isEquals();
  }

  public int hashCode() {
    return new HashCodeBuilder(17, 37).
        append(file).
        append(metric).
        append(rulesCategory).
        append(rule).
        append(subkey).
        toHashCode();
  }

  public Object clone() {
    return new MeasureKey(this.metric, this.rulesCategory, this.rule, this.file, this.subkey);
  }

  public File getFile() {
    return file;
  }

  public MeasureKey setFile(File file) {
    this.file = file;
    return this;
  }

  public Metric getMetric() {
    return metric;
  }

  public MeasureKey setMetric(Metric metric) {
    this.metric = metric;
    return this;
  }

  public RulesCategory getRulesCategory() {
    return rulesCategory;
  }

  public MeasureKey setRulesCategory(RulesCategory rulesCategory) {
    this.rulesCategory = rulesCategory;
    return this;
  }

  public Rule getRule() {
    return rule;
  }

  public MeasureKey setRule(Rule rule) {
    this.rule = rule;
    return this;
  }

  public String getSubkey() {
    return subkey;
  }

  public void setSubkey(String subkey) {
    this.subkey = subkey;
  }
}