/*
 * This program is copyright (c) 2007 Hortis-GRC SA.
 * 
 * This file is part of Sonar.
 * Sonar is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * Sonar is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with Sonar; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
package ch.hortis.sonar.service;

import ch.hortis.sonar.model.MavenProject;
import ch.hortis.sonar.model.Metric;
import ch.hortis.sonar.model.Metrics;
import ch.hortis.sonar.model.ProjectMeasure;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ProjectMeasureService extends Service {
  
  public ProjectMeasureService( EntityManager manager ) {
    super( manager );
  }
  
  public Map<MeasureKey, List<ProjectMeasure>> getHistory(MavenProject project, Metrics[] metrics) {
    MetricService metricsService = new MetricService( manager );
    List<Metric> targets = new ArrayList<Metric>();
    for ( Metrics target : metrics ) {
      targets.add( metricsService.getMetric( target ) );
    }
    
    Query query = manager.createNamedQuery( ProjectMeasure.SQL_SELECT_HISTORY );
    query.setParameter( "project", project);
    query.setParameter( "metrics", targets );

    HashMap<MeasureKey, List<ProjectMeasure>> result = new HashMap<MeasureKey, List<ProjectMeasure>>();
    List<ProjectMeasure> measures = query.getResultList();
    for(ProjectMeasure measure : measures) {
      MeasureKey key = new MeasureKey(measure);
      List<ProjectMeasure> sublist = result.get(key);
      if (sublist == null) {
        sublist = new ArrayList<ProjectMeasure>();
        result.put(key, sublist);
      }
      sublist.add(measure);
    }
    return result;
  }
}
