/*
 * This program is copyright (c) 2007 Hortis-GRC SA.
 * 
 * This file is part of Sonar.
 * Sonar is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * Sonar is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with Sonar; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
package ch.hortis.sonar.service;

import ch.hortis.sonar.model.File;
import ch.hortis.sonar.model.Rule;
import ch.hortis.sonar.model.RuleFailure;
import ch.hortis.sonar.model.RuleFailureLevel;
import ch.hortis.sonar.model.RulesCategory;
import ch.hortis.sonar.model.Snapshot;

import javax.persistence.EntityManager;
import java.util.List;

public class RuleFailureService extends Service {

  public RuleFailureService(EntityManager manager) {
    super(manager);
  }
  
  public Long getRuleFailuresCount( Snapshot snapshot, RuleFailureLevel ruleFailureLevel ) {
    long count = 0;
    List<RuleFailure> failures = snapshot.getRuleFailures();
    for ( RuleFailure failure : failures ) {
      if ( failure.getFile() == null &&
           failure.getLevel() == ruleFailureLevel ) {
        count++;
      }
    }
    return count;
  }
  
  public Long getRuleFailuresCount( Snapshot snapshot, File file, RuleFailureLevel ruleFailureLevel ) {
    long count = 0;
    List<RuleFailure> failures = snapshot.getRuleFailures();
    for ( RuleFailure failure : failures ) {
      if ( failure.getFile() != null &&
          failure.getFile().getId().equals( file.getId() ) &&
           failure.getLevel() == ruleFailureLevel ) {
        count++;
      }
    }
    return count;
  }
  
  public Long getRuleFailuresCount( Snapshot snapshot, RulesCategory category, RuleFailureLevel ruleFailureLevel ) {
    long count = 0;
    List<RuleFailure> failures = snapshot.getRuleFailures();
    for ( RuleFailure failure : failures ) {
      if ( failure.getFile() == null &&
          failure.getRule().getRulesCategory().getId().equals( category.getId() ) &&
           failure.getLevel() == ruleFailureLevel ) {
        count++;
      }
    }
    return count;
  }
  
  public Long getRuleFailuresCount( Snapshot snapshot, Rule rule, RuleFailureLevel ruleFailureLevel ) {
    long count = 0;
    List<RuleFailure> failures = snapshot.getRuleFailures();
    for ( RuleFailure failure : failures ) {
      if ( failure.getFile() == null &&
          failure.getRule().getId().equals( rule.getId() ) && 
           failure.getLevel() == ruleFailureLevel ) {
        count++;
      }
    }
    return count;
  }

}
